/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <wchar.h>

#include <fstream>

#include <stdlib.h>
#include <getopt.h>

#include "libtkcommon/config.h"
#include "libtkrandom/bit_printer.h"

#define _(x) x

void
print_help ()
{
  std::cout  << PACKAGE_NAME << " version " << PACKAGE_VERSION << std::endl;
  std::wcout << L"Copyright (C) 2005 Robert Ham (rah@bash.sh)" << std::endl;
  std::wcout << std::endl;
  std::wcout << L"This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it" << std::endl;
  std::wcout << L"under the terms of the GNU General Public License, version 3 or later.  See" << std::endl;
  std::wcout << L"the COPYING file that came with this software for details." << std::endl;
  std::wcout << std::endl;
  std::wcout << L"Usage: bit-printer [options] <file> [ <file> ... ]" << std::endl;
  std::wcout << std::endl;
  std::wcout << L" -h, --help                  " << _("Display this help info") << std::endl;
  std::wcout << L" -t, --type                  " << _("Stream character type (char, wchar_t, schar)") << std::endl;
  std::wcout << std::endl;
}

typedef enum
  {
    CHAR,
    WCHAR_T,
    SIGNED_CHAR
  } char_type_t;

template <typename C>
static void
print_file(const Tk::String &file_name, unsigned line_len)
{
  std::cout << "===== " << static_cast<std::string>(file_name) << " =====" << std::endl;
  Tk::Random::BitPrinter<C, char> bit_printer("", std::cout, line_len);
  
  std::basic_ifstream<char> in;
  in.exceptions(std::ios_base::badbit);
  in.open(static_cast<std::string>(file_name).c_str());
  
  Tk::BasicStream<C> *conv_stream( new Tk::BasicStream<C>(in) );
  
  C c;
  while ( conv_stream->get_stream()->get(c) )
    bit_printer.print_bits(c);
  
  in.close();
  delete conv_stream;
}

int
main (int argc, char ** argv)
{
  char_type_t char_type(CHAR);
  
  int opt;
  const char * options = "ht:";
  struct option long_options[] = {
    { "help",        no_argument,       NULL, 'h' },
    { "type",        required_argument, NULL, 't' },
    { 0, 0, 0, 0 }
  };
  
  while ((opt = getopt_long (argc, argv, options, long_options, NULL)) != -1) {
    switch (opt)
      {
      case 't':
	{
	  std::string o(optarg);
	    if (o == "char")
	      char_type = CHAR;
	    else if (o == "wchar_t")
	      char_type = WCHAR_T;
	    else if (o == "schar")
	      char_type = SIGNED_CHAR;
	    else
	      {
		std::cout << _("Unknown type") << " `" << optarg << "'" << std::endl;
		print_help();
		exit(1);
	      }
	  break;
	}

      case 'h':
	print_help ();
	exit (0);
	break;
      case ':':
      case '?':
	print_help ();
	exit (1);
	break;
      }
  }
  
  for (int i = optind; i < argc; ++i)
    switch (char_type)
      {
      case CHAR:        print_file<char>(argv[i], 80); break;
      case WCHAR_T:     print_file<wchar_t>(argv[i], 96); break;
      case SIGNED_CHAR: print_file<signed char>(argv[i], 80); break;
      }
  
  exit (0);
}
