/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdexcept>

#include "editor/domain.h"
#include "libtkdb/domain.h"

using namespace Tk::Editor;

Domain:: Domain()
  : _store( Gtk::ListStore::create(_columns) )
{
  WDEBUG_OUT << L"initialising Domain" << std::endl;

  set_model(_store);
  pack_start(_columns._label);
  
  for (Tk::Db::DomainMap::const_iterator iter( Tk::Db::domains.begin() );
       iter != Tk::Db::domains.end();
       ++iter)
    {
      Gtk::TreeModel::iterator row_iter( _store->append() );
      Gtk::TreeModel::Row row(*row_iter);
      row[_columns._name]  = iter->first;
      row[_columns._label] = static_cast<std::string>(iter->second);
    }
  
  set_active(0);

  WDEBUG_OUT << L"initialised Domain" << std::endl;
}

Tk::String
Domain:: get_domain() const
{
  return (*get_active())[_columns._name];
}

Domain::SignalDomainChanged
Domain:: signal_domain_changed()
{
  return _signal_domain_changed;
}

void
Domain:: on_changed()
{
  _signal_domain_changed.emit(get_domain());
}
