/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2006 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_GNUPLOT_H__
#define __TK_GNUPLOT_H__

#include <wchar.h>
#include <iostream>
#include <ext/stdio_filebuf.h>

#include "analyser.h"

namespace Tk {
  namespace Analysis {
    
    class Gnuplot
      {
      private:
	typedef std::string Argument;
	typedef std::list<Argument> Arguments;
	typedef __gnu_cxx::stdio_filebuf<wchar_t> FileStream;
	typedef std::basic_ostream<wchar_t> OutputStream;
	
	FileStream   * _gnuplot_file;
	OutputStream * _gnuplot;
	
	static Arguments arguments(unsigned set_count, unsigned set_size);
	
	void spawn (const Arguments &arguments);
	void plot(const String &prefix, const OffsetList &offsets, bool first, bool last);
	void position_counts(const PositionCounts &position_counts);
	void position_function(const PositionCounts &position_counts,
			       const PositionFunction &positions,
			       const Count &position_count_max,
			       const Probability &probability);
	void offsets(const OffsetStatisticsSet &offset_stats);
	void statistics(const OffsetStatisticsSet &offset_stats);
	
	
      public:
	Gnuplot(const String &name, const OffsetStatisticsSet &stats);
	Gnuplot(const String &name_a, const OffsetStatisticsSet &stats_a,
		const String &name_b, const OffsetStatisticsSet &stats_b);
	~Gnuplot();
      };
    
  }; /* namespace Analysis */
}; /* namespace Tk */

#endif /* __TK_GNUPLOT_H__ */

