/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <fstream>
#include <algorithm>


//#define TK_STREAM_DEBUG
#include "libtkcommon/stream.h"
#include "libtkdb/bytes.h"
#include "libtkrandom/bit_printer.h"

using namespace Tk::Db;

Bytes:: Bytes()
{
}

Bytes:: Bytes(const Bytes &other)
  : odbc::Bytes(other)
{
}

Bytes:: Bytes(const ODBCXX_SIGNED_CHAR_TYPE *data, size_t dataLen)
  : odbc::Bytes(data, dataLen)
{
}

Bytes:: Bytes(const Tk::String &file_name)
{
  std::basic_ifstream<char> in;
  in.exceptions(std::ios_base::badbit);
  in.open(static_cast<std::string>(file_name).c_str());
  
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE> *conv_stream( new Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE>(in) );
  
  std::vector<ODBCXX_SIGNED_CHAR_TYPE> gotten;
  ODBCXX_SIGNED_CHAR_TYPE c;
  while ( conv_stream->get_stream()->get(c) )
    gotten.push_back(c);
  
  delete conv_stream;
  in.close();
  
  *this = Bytes(&gotten[0], gotten.size());
}

Bytes:: Bytes(std::istream &stream, std::string::size_type *len)
{
  Tk::Stream *base_strm;
  if (len)
    base_strm = new Tk::Stream(stream, *len);
  else
    base_strm = new Tk::Stream(stream);
  
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE> *conv_strm( base_strm->write_stream<ODBCXX_SIGNED_CHAR_TYPE>() );
  delete base_strm;
  
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE>::len_type conv_len( conv_strm->get_len() );
  ODBCXX_SIGNED_CHAR_TYPE conv_buf[conv_len];
  conv_strm->get_stream()->read(conv_buf, conv_len);
  delete conv_strm;
  
#ifdef TK_STREAM_DEBUG
  DEBUG_OUT << "Final converted data buffer length: " << conv_len << std::endl;
#endif
  *this = Bytes(conv_buf, conv_len);
}


Bytes:: ~Bytes()
{
  for (StreamList::iterator iter( _gotten_streams.begin() );
       iter != _gotten_streams.end();
       ++iter)
    delete *iter;
}

Tk::Stream *
Bytes:: get_stream()
{
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE> base_strm(getData(), getSize());
  Tk::Stream *stream( base_strm.write_stream<char>() );
  _gotten_streams.push_back(stream);
  return stream;
}

Tk::Random::Bits
Bytes:: get_bits(unsigned long bit_count) const
{
  static const unsigned short BITS_PER_CHAR( Tk::Random::bits_per_char<ODBCXX_SIGNED_CHAR_TYPE>() );
  
  const ODBCXX_SIGNED_CHAR_TYPE *data( getData() );
  size_t len( getSize() );
  
  Tk::Random::Bits bits;
  bool bit;
  ODBCXX_SIGNED_CHAR_TYPE c;
  for (size_t i(0); i < len && bit_count > 0; ++i)
    {
      c = data[i];
      for (unsigned short char_index(0); (char_index < BITS_PER_CHAR) && (bit_count > 0); ++char_index)
	{
	  bit = (c & (1 << (BITS_PER_CHAR - char_index - 1))) != 0;
	  bits.push_back(bit);
	  --bit_count;
	}
    }
  
  return bits;
}

void
Bytes:: write_to_file(const Tk::String &file_name) const
{
#ifdef TK_STREAM_DEBUG
  DEBUG_OUT << "Writing " << getSize() << " characters from database" << std::endl;
#endif
  Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE> *bytes_stream( new Tk::BasicStream<ODBCXX_SIGNED_CHAR_TYPE>(getData(), getSize()) );
#ifdef TK_STREAM_DEBUG
  DEBUG_OUT << "Converted to byte stream with " << bytes_stream->get_len() << " characters" << std::endl;
#endif
  
  Tk::Stream *file_stream( bytes_stream->write_stream<char>() );
#ifdef TK_STREAM_DEBUG
  DEBUG_OUT << "Converted to file stream with " << file_stream->get_len() << " characters" << std::endl;
#endif
  delete bytes_stream;
  
  std::ofstream out(static_cast<std::string>(file_name).c_str());
  
  out.exceptions(std::ios_base::badbit);
  /*
  std::copy(std::istream_iterator<char>(*file_stream->get_stream()),
	    std::istream_iterator<char>(),
	    std::ostream_iterator<char>(out));
  */
  Tk::Stream::istream_type *file_stdstream( file_stream->get_stream() );
  char c;
#ifdef TK_STREAM_DEBUG
  unsigned out_count(0);
#endif
  while (file_stdstream->read(&c, 1))
    {
      out.write(&c, 1);
#ifdef TK_STREAM_DEBUG
      ++out_count;
#endif
    }
  
#ifdef TK_STREAM_DEBUG
  DEBUG_OUT << "Finished writing "
	    << out_count
	    << " characters to file (stream is "
	    << (file_stream->get_stream()->bad() ? "bad" : "good")
	    << ")"
	    << std::endl;
#endif
  
  
  delete file_stream;
  out.close();
}
