/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DB_DATABASE_H__
#define __TK_DB_DATABASE_H__

#include <glibmm.h>

#include "libtkdb/connection.h"
#include "libtkdb/report.h"
#include "libtkdb/value.h"
#include "libtkdb/experiment_result.h"

namespace Tk {
  namespace Db {
    
    class Database
      {
	/* async operation */
      public:
	typedef sigc::signal<void, const Report &> SignalReport;
	typedef sigc::signal<void, const ReportUpdate &> SignalReportUpdate;
	
      protected:
	typedef std::list<Report *> Reports;
	typedef std::list<ReportUpdate *> ReportUpdates;
	
	Connection::Mutex _reporting_lock;
	Reports _running_reports;
	Reports _pending_reports;
	ReportUpdates _report_updates;
	
	Glib::Dispatcher _report_dispatcher;
	SignalReport _signal_report;
	SignalReportUpdate _signal_report_update;
	
      private:
	void send_report(Report *report);
	void send_report_update(ReportUpdate *update);
	
	void postpone_reporting();
	bool receive_reporting();
	void receive_report(Report *report);
	void receive_report_update(ReportUpdate *update);
	
	/* db stuff */
      private:
	Connection  _connection;
	
	Id insert_property(const Id &property_type_id);
	Id insert_property_value (const Id &property_id);
	void insert_property_value_data (const Id &property_value_id, const Tk::String &table_name, Data::Value &value);
	void update_property_value_data (const Id &property_value_id, const Tk::String &table_name, Data::Value &value);
	void insert_domain_property(const Tk::String &domain, const Id &domain_id, const Id &property_id);
	
	Id insert_experiment(const Id &location_id);
	void experiment_insert_operators(const Id &experiment_id, const Ids &operators, const Report *parent_report);
	void experiment_insert_result(const Id &experiment_id, const Tk::Resolver::Result &result, const Report *parent_report);
	
	void insert_experiment_run(ExperimentResult *experiment);
	
      public:
	Database (const Tk::String &data_source_name = Tk::String("TkResolver++"),
		  const Tk::String &username = Tk::String("psi"),
		  const Tk::String &password = Tk::String("isp"));

	SignalReport signal_report();
	SignalReportUpdate signal_report_update();
	
	Connection *   get_connection();
	Ids            get_ids(PStatement &stmt);
	void           property_type_get_details (const Tk::String &property_domain,
						  const Tk::String &property_name,
						  Id *property_type_id = 0,
						  Tk::String *property_value_type_name = 0,
						  Tk::String *property_value_table_name = 0);
	
	Id             lookup_location(const Tk::String &location_name);
	Id             choose_location();
	Id             choose_operator();
	Code           choose_intent(unsigned trace_count, const Tk::String &algorithm_spec, bool test);
	IntentInfoList get_intent_options(unsigned trace_count);
	
	Id             create_entity(const Tk::String &domain);
	Ids             get_entities(const Tk::String &domain);
	Id             find_property(const Tk::String &domain, const Id &entity_id, const Tk::String &property_name);
	Data::Value     get_property(const Tk::String &domain, const Id &entity_id, const Tk::String &property_name);
	Id              set_property(const Tk::String &domain, const Id &entity_id, const Tk::String &property_name, Data::Value &value);
	Data::TypeMap  get_domain_property_types(const Tk::String &domain);
	Data::ValueMap get_properties(const Tk::String &domain, const Id &entity_id);
	
	void           insert_experiment(ExperimentResult *experiment);
	
	// Lock connection around this
	DataSet *      get_results(const Id &experiment);
    };

  }; /* namespace Db */
}; /* namespace Tk */

#endif /* __TK_DB_DATABASE_H__ */
