/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DB_REPORT_H__
#define __TK_DB_REPORT_H__

#include "libtkcommon/string.h"

namespace Tk {
  namespace Db {
    
    class Report
    {
    private:
      static unsigned  _next_id;
      unsigned         _id;
      const Report    *_parent;
      Tk::String       _description;
      
    public:
      Report(const Tk::String &description, const Report *parent = 0);
      const Report * get_parent() const;
      const Tk::String & get_description() const;
      bool operator ==(const Report &other) const;
    };
    
    class ReportUpdate
    {
    public:
      enum Status
	{
	  Running,
	  Ended
	};
      
    private:
      const Report  &_report;
      Status         _status;
      double         _fraction;
      
    public:
      ReportUpdate(const Report &report, Status status, double fraction = -1.0);
      const Report &get_report() const;
      Status get_status() const;
      double get_fraction() const;
    };
    

  }; /* namespace Db */
}; /* namespace Tk */
    


#endif /* __TK_DB_REPORT_H__ */
