/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DB_UI_DOMAIN_SELECTOR_H__
#define __TK_DB_UI_DOMAIN_SELECTOR_H__

#include <gtkmm.h>

#include "libtkdb/database.h"

namespace Tk {
  namespace Db {
    namespace Ui {
      
      class NoSelection : public std::runtime_error
      {
      public:
	NoSelection();
      };
      
      
      class DomainSelector : public Gtk::TreeView
      {
      protected:
	Db::Database                         *_database;

	virtual Tk::String render_entity(const Tk::Db::Id &entity_id) = 0;
	
      private:
	class Columns : public Gtk::TreeModelColumnRecord
	{
	public:
	  Gtk::TreeModelColumn<Tk::Db::Id> id;
	  Gtk::TreeModelColumn<Tk::StringValue> rendering;
	  Columns() { add(id); add(rendering); }
	};
	
	Tk::String                            _domain;
	
	Columns                               _columns;
	Glib::RefPtr<Gtk::ListStore>          _store;
	
	Gtk::CellRendererText   _renderer;
	Gtk::TreeViewColumn     _column;
	
	Gtk::TreeViewColumn::SlotCellData _cell_data_slot;
	void cell_data(Gtk::CellRenderer *cell, const Gtk::TreeModel::iterator &iter);
	void add_selected(const Gtk::TreeModel::iterator &iter, Tk::Db::Ids &ids);
	
      public:
	DomainSelector(Db::Database *database,
		       const Tk::String &domain,
		       bool multiple_selection = false);
	
	Tk::Db::Ids get_selected();
      };
      
    }; /* namespace Ui */
  }; /* namespace Db */
}; /* namespace Tk */




#endif /* __TK_DB_UI_DOMAIN_SELECTOR_H__ */
