/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkdbui/file_value.h"

using namespace Tk::Db::Ui;


FileValue:: FileValue(const Tk::Db::Bytes &bytes)
  : _bytes(bytes),
    Value(Tk::Db::Data::Blob)
{
  Gtk::Button *button = Gtk::manage( new Gtk::Button(Gtk::Stock::OPEN) );
  pack_start(*button, false, false);
  button->signal_clicked().connect( sigc::mem_fun(*this, &FileValue::load) );

  button = Gtk::manage( new Gtk::Button(Gtk::Stock::SAVE) );
  pack_start(*button, false, false);
  button->signal_clicked().connect( sigc::mem_fun(*this, &FileValue::save) );
}

static Tk::String *
get_filename(Gtk::FileChooserAction action, const Gtk::StockID& action_stock_id)
{
  Gtk::FileChooserDialog *dialog( new Gtk::FileChooserDialog("Select file", action) );
  dialog->add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog->add_button(action_stock_id, Gtk::RESPONSE_ACCEPT);
  dialog->set_default_response(Gtk::RESPONSE_ACCEPT);
  dialog->set_local_only(true);
  
  int response( dialog->run() );
  Tk::String file_name( dialog->get_filename() );
  delete dialog;
  
  if (response != Gtk::RESPONSE_ACCEPT)
    return 0;
  
  return new Tk::String(file_name);
}

void
FileValue:: load()
{
  Tk::String *filename( get_filename(Gtk::FILE_CHOOSER_ACTION_OPEN, Gtk::Stock::OPEN) );
  if (filename)
    {
      _bytes = Tk::Db::Bytes(*filename);
      delete filename;
      value_changed();
    }
}

void
FileValue:: save()
{
  Tk::String *filename( get_filename(Gtk::FILE_CHOOSER_ACTION_SAVE, Gtk::Stock::SAVE) );
  if (filename)
    {
      _bytes.write_to_file(*filename);
      delete filename;
    }
}

Tk::Db::Data::Value
FileValue:: get_value() const
{
  return Tk::Db::Data::Value(_bytes);
}
