/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RANDOM_BIT_PRINTER_H__
#define __TK_RANDOM_BIT_PRINTER_H__

#include "libtkcommon/string.h"
#include "libtkrandom/types.h"

namespace Tk {
  namespace Random {
    
    template <typename C, typename OC = wchar_t>
    class BitPrinter
    {
    public:
      typedef std::basic_string<OC> String;
      typedef std::basic_ostringstream<OC> OStringStream;
      typedef std::basic_ostream<OC> OutputStream;
    private:
      static const unsigned short BITS_PER_CHAR =
	std::numeric_limits<C>::digits + static_cast<unsigned>(std::numeric_limits<C>::is_signed);
    
      const String   _prefix;
      const unsigned _line_len;
      unsigned       _out_count;
      unsigned       _line_count;
      OStringStream *_stream;
      OutputStream  &_output;
      
      void print_line()
      {
	static const String bits_line(Tk::String("Bits line\t"));
	static const String colon(Tk::String(": "));

	_output << _prefix << bits_line << _line_count << colon << _stream->str() << std::endl;
	
	delete _stream;
	_stream = new OStringStream;
      }
      
    public:
      BitPrinter(const String &prefix, OutputStream &output, unsigned line_len = 80)
	: _prefix(prefix),
	  _line_len(line_len),
	  _out_count(0),
	  _line_count(0),
	  _stream( new OStringStream ),
	  _output(output)
      {
      }
      
      ~BitPrinter()
      {
	if (_out_count > 0)
	  print_line();
	delete _stream;
      }
      
      void print_bits(C bits)
      {
	for (unsigned i(0); i < BITS_PER_CHAR; ++i)
	  {
	    static const String zero(Tk::String("0"));
	    static const String one(Tk::String("1"));
	    
	    (*_stream) << (((bits & (1 << (BITS_PER_CHAR - i - 1))) != 0) ? one : zero);
	    
	    if (_out_count == _line_len - 1)
	      {
		print_line();
		_out_count = 0;
		++_line_count;
	      }
	    else
	    ++_out_count;
	  }
	
	if (_out_count > 0)
	{
	  static const String space(Tk::String(" "));
	  (*_stream) << space;
	}
      }
    };
    
  }; /* namespace Random */
}; /* namespace Tk */


#endif /* __TK_RANDOM_BIT_PRINTER_H__ */
