/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

//#define TK_STREAM_DEBUG

#include "libtkrandom/bits.h"
#include "libtkrandom/bit_printer.h"

using namespace Tk::Random;

BitStream *
Bits:: get_data () const
{
  std::basic_stringstream<BitChar> data;
  data.exceptions(std::ios_base::badbit);
  
  unsigned bit_count = size();
  
  unsigned char_index(0);
  BitChar c(0);
#ifdef TK_STREAM_DEBUG
  Tk::Random::BitPrinter<BitChar, wchar_t> printer(Tk::String(__FUNCTION__) + L" ", WDEBUG_STRM);
#endif
  for (const_iterator iter( begin() );
       iter != end();
       ++iter)
    {
      c |= *iter << (BITS_PER_CHAR - char_index - 1);
      
      if (char_index == BITS_PER_CHAR - 1)
	{
	  data.write(&c, 1);
#ifdef TK_STREAM_DEBUG
	  printer.print_bits(c);
#endif
	  c = 0;
	  char_index = 0;
	}
      else
	++char_index;
    }
  
  if (char_index > 0)
    {
      data.write(&c, 1);
#ifdef TK_STREAM_DEBUG
      printer.print_bits(c);
#endif
    }
  
  return new BitStream(data);
}

Bits::size_type
Bits:: get_data_len() const
{
  size_type bit_count( size() );
  return (bit_count / BITS_PER_CHAR) + ((bit_count % BITS_PER_CHAR > 1) ? 1 : 0);
}

static void
print_line(std::ostringstream **bits_line_ptr, unsigned line_count)
{
  WDEBUG_OUT << L"Bits line\t" << line_count << L": " << Tk::String((*bits_line_ptr)->str()) << std::endl;
  
  delete *bits_line_ptr;
  *bits_line_ptr = new std::ostringstream;
}

void
Bits:: dump_bits() const
{
  
  const unsigned line_len(40);
  unsigned out_count(0);
  unsigned line_count(0);
  std::ostringstream *bits_line( new std::ostringstream );
  
  WDEBUG_OUT << L"Total bits: " << size() << std::endl;
  for (const_iterator bits_iter(begin());
       bits_iter != end();
       ++bits_iter)
    {
      (*bits_line) << (*bits_iter ? "1" : "0");
      
      if (out_count == line_len - 1)
	{
	  print_line(&bits_line, line_count);
	  out_count = 0;
	  ++line_count;
	}
      else
	++out_count;
      
      if (out_count % 8 == 0)
	(*bits_line) << ' ';
    }
  
  WDEBUG_STRM.flush();
  
  delete bits_line;
}
