/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <unistd.h>

#include <iostream>
#include <cmath>

#include "libtkcommon/exceptions.h"
#include "libtkcommon/debug.h"
#include "libtkcommon/intl.h"
#include "libtkcommon/maths.h"
#include "libtkcommon/real_time.h"
#include "libtkrandom/randomness.h"
#include "libtkrandom/retriever_registry.h"
#include "libtkrandom/types.h"

using namespace Tk::Random;

Randomness:: Randomness(bool record_use)
  : _bit_index(BITS_PER_CHAR),
    _used_bits(record_use ? new Bits : 0)
{
}

Randomness:: ~Randomness()
{
  if (_used_bits)
    delete(_used_bits);
}

void
Randomness:: add_buffer(const Buffer &buffer)
{
  _buffer.insert<Buffer::const_reverse_iterator>(_buffer.begin(), buffer.rbegin(), buffer.rend());
}

bool
Randomness:: get_bit()
{
  if (_bit_index >= BITS_PER_CHAR)
    {
      if (_buffer.empty())
	throw OutOfBits();
      
      _bits = _buffer.back();
      _bit_index = 0;
      _buffer.pop_back();
    }
  
  
  bool bit( _bits & (1 << (BITS_PER_CHAR - _bit_index - 1)) );
  
  if (_used_bits)
    _used_bits->push_back(bit);
  
  ++_bit_index;
  
  return bit;
}

const Bits *
Randomness:: get_used_bits() const
{
  return _used_bits;
}


/*
unsigned long
Randomness:: get_number(unsigned long upper, const String &retriever_type)
{
  static const unsigned short BITS_PER_UL( std::numeric_limits<unsigned long>::digits );
  unsigned long bits = 0;

  Randomness * randomness = new Randomness(retriever_type, true, BITS_PER_UL / BITS_PER_CHAR, false);
  
  
  for (unsigned short i = 0; i < BITS_PER_UL; ++i)
    {
      if (randomness->get_bit())
	bits |= (static_cast<unsigned long>(1) << i);
    }

  delete randomness;

  
  static const unsigned long max( std::numeric_limits<unsigned long>::max() );
  static const long double max_divisor( static_cast<long double>(max) + static_cast<long double>(1.0) );
  long double bits_ratio( static_cast<long double>(bits) / max_divisor );
  long double d_value( static_cast<long double>(upper) * bits_ratio );
  return static_cast<unsigned long>(d_value);
}
*/


/* EOF */
