/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RESOLVER_COLOUR_H__
#define __TK_RESOLVER_COLOUR_H__

#include <vector>

#include <SDL/SDL.h>

namespace Tk {
  namespace Resolver {
  
    typedef Uint32 SDLColour;
  
    class Colour
    {
    private:
      typedef Uint8 Component;
      
      double _h;
      double _s;
      double _v;
      
      Component p() const;
      Component q(double f) const;
      Component t(double f) const;
      
      void get_rgb(Component &r, Component &g, Component &b) const;
      
    public:
      Colour(double h, double s, double v);
      void increment(const Colour &target, unsigned divisor);
      double get_h() const;
      double get_s() const;
      double get_v() const;
      SDLColour map_rgb(SDL_PixelFormat *fmt) const;
    };
  
    typedef std::vector<Colour> Colours;
  
    class ColourGenerator
    {
    private:
      static Colour generate();
      
      unsigned _screen_height;
      
      Uint32    _start;
      unsigned  _line_count;
      Colour    _current;
      unsigned  _lines_left;
      Colour    _target;
    
    public:
      ColourGenerator(unsigned screen_height);
      Colour get_colour();
    };
    
  }; /* namespace Resolver */
}; /* namespace Tk */

#endif /* __TK_RESOLVER_COLOUR_H__ */
